using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Data;
using System.Windows.Forms;
using iAnywhere.Data.UltraLite;

namespace SchemaUpgrade.DataAccess
{
    struct SchemaChangeInfo
    {
        public String schemaChangeStatement;
        public Int32 sqlID;
    }
    
    class DataAccess : IDisposable
    {
        private ULConnection ulConnection;
        private static DataAccess instance;

        // constructor
        private DataAccess()
        {
            try
            {
                ulConnection = new ULConnection(@"dbf=SchemaUpgrade.udb");
                ulConnection.Open();
            }
            catch (ULException uex)
            {
                Debug.Print(uex.Message);
            }
        }

        public static DataAccess Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new DataAccess();
                }
                return instance;
            }
        }

        public void Dispose()
        {
            //dc.Dispose();
        }

        public ULResultSet EmployeeResultSet()
        {
            try
            {
                ULResultSet rs;
                using (ULCommand cmd = ulConnection.CreateCommand())
                {
                    if (this.CurrentSchemaVersion() == 1)
                    {
                        cmd.CommandText = @"SELECT EmployeeID, EmployeeName FROM Employee";
                    }
                    else if (this.CurrentSchemaVersion() == 2)
                    {
                        cmd.CommandText = @"SELECT EmployeeID, EmployeeName, EmployeePhone FROM Employee";
                    }
                    else if (this.CurrentSchemaVersion() == 3)
                    {
                        cmd.CommandText = @"SELECT EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail FROM Employee";
                    }
                    rs = cmd.ExecuteResultSet();
                }
                return rs;
            }
            catch (Exception ex)
            {
                Debug.Print(ex.Message);
                throw new Exception();
            }
        }

        public Int32 CurrentSchemaVersion()
        {
            Int32 DEFAULT_VERSION = 0;
            try
            {
                using (ULCommand cmd = ulConnection.CreateCommand())
                {
                    cmd.CommandText = @"SELECT CurrentVersion FROM SU_RemoteDatabaseVersion";
                    return (Int32)cmd.ExecuteScalar();
                }
            }
            catch (Exception ex)
            {
                Debug.Write(ex.Message);
                return DEFAULT_VERSION;
            }
        }

        public Int32 TargetSchemaVersion()
        {
            Int32 DEFAULT_VERSION = 1;
            try
            {
                using (ULCommand cmd = ulConnection.CreateCommand())
                {
                    cmd.CommandText = @"SELECT TargetVersion FROM SU_RemoteDatabaseVersion";
                    return (Int32)cmd.ExecuteScalar();
                }
            }
            catch (Exception ex)
            {
                Debug.Write(ex.Message);
                return DEFAULT_VERSION;
            }
        }

        public Boolean IsSchemaUpgradeComplete()
        {
            try
            {
                Int32 Result;
                using (ULCommand cmd = ulConnection.CreateCommand())
                {
                    cmd.CommandText = @"SELECT (TargetVersion - CurrentVersion) FROM SU_RemoteDatabaseVersion";
                    Result = (Int32)cmd.ExecuteScalar();
                    if (Result == 0)
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
            }
            catch (ULException uex)
            {
                Debug.Print(uex.Message);
                return false;
            }
            catch (Exception ex)
            {
                Debug.Print(ex.Message);
                return false;
            }
        }

        private SchemaChangeInfo[] SchemaChangeInfo(Int32 currentVersion, Int32 targetVersion)
        {
            int i;
            int statementCount;
            SchemaChangeInfo[] sci;
            using (ULCommand cmd = ulConnection.CreateCommand())
            {
                cmd.CommandText = @"SELECT SqlId, SqlStatement FROM SU_SQLStatement" +
                    " WHERE CurrentVersion = " + currentVersion.ToString() +
                    " AND TargetVersion = " + targetVersion.ToString()  +
                    " ORDER BY Sequence ASC";
                using (ULResultSet rs = cmd.ExecuteResultSet())
                {
                    statementCount = rs.RowCount;
                    sci = new SchemaChangeInfo[statementCount];

                    rs.MoveBeforeFirst();
                    i = -1;
                    while (rs.MoveNext())
                    {
                        i++;
                        sci[i].sqlID = rs.GetInt32(0);
                        sci[i].schemaChangeStatement = rs.GetString(1);
                    }
                }
            }
            return sci;
        }

        private Boolean ExecuteSchemaChangeStatement(SchemaChangeInfo sci)
        {
            int schemaRowsAffected;

            try
            {
                using (ULCommand schemaCommand = ulConnection.CreateCommand())
                {
                    try
                    {
                        schemaCommand.CommandText = sci.schemaChangeStatement;
                        schemaRowsAffected = schemaCommand.ExecuteNonQuery();
                    }
                    catch (ULException uex)
                    {
                        Debug.Print(uex.Message);
                        return false;
                    }
                }
                return true;
            }
            catch (Exception ex)
            {
                Debug.Print(ex.Message);
                return false;
            }
        }

        public Boolean UpgradeSchema()
        {
            // Get the version to upgrade to
            Int32 iStep;
            Int32 currentVersion = this.CurrentSchemaVersion();
            Int32 targetVersion = this.TargetSchemaVersion();

            SchemaChangeInfo[] sci;

            sci = this.SchemaChangeInfo(currentVersion, targetVersion);
            for (iStep = 0; iStep < sci.Length; iStep++)
            {
                if (!this.ExecuteSchemaChangeStatement(sci[iStep]))
                {
                    // although ReverseSchemaUpgrade does provide return information,
                    // it is not used in this application.
                    this.ReverseSchemaUpgrade(currentVersion, targetVersion, iStep);
                    return false;
                }
            }
            using (ULCommand cmd = ulConnection.CreateCommand())
            {
                cmd.CommandText = @"UPDATE SU_RemoteDatabaseVersion SET CurrentVersion = TargetVersion";
                cmd.ExecuteNonQuery();
            }
            return true;
            
        }

        /// <summary>
        /// Reverse the upgrade from currentVersion to targetVersion
        /// </summary>
        /// <param name="currentVersion">the schema version at the start of the failed upgrade</param>
        /// <param name="targetVersion">the schema version that the upgrade was supposed to move to</param>
        /// <param name="iStep">the 0-indexed step at which the upgrade failed</param>
        private Boolean ReverseSchemaUpgrade( Int32 currentVersion, Int32 targetVersion, Int32 iStepFailed){
            SchemaChangeInfo[] sci;
            // get SchemaChangeInfo with the targetVersion first (ie, the downgrade)
            Int32 iStep;
            sci = this.SchemaChangeInfo(targetVersion, currentVersion );
            for (iStep = sci.Length - iStepFailed - 1; iStep < sci.Length; iStep++)
            {
                if (!this.ExecuteSchemaChangeStatement(sci[iStep]))
                {
                    return false;
                }
            }
            return true;
        }

        public Boolean Synchronize( String publication, String version )
        {
            try
            {
                ULSyncParms _syncParms = ulConnection.SyncParms;
                _syncParms.Stream = ULStreamType.TCPIP;
                _syncParms.PublicationMask = ulConnection.Schema.GetPublicationSchema(publication).Mask;
                _syncParms.Version = version;
                _syncParms.UserName = "user1";
                _syncParms.Password = "pwd_user1";
                ulConnection.Synchronize();
                return true;
            }
            catch (ULException uex)
            {
                MessageBox.Show(uex.Message);
                return false;
            }
        }
    }
}
