message '  ---- Creating tables...' type info to client
go

-- This is the table with data that needs to be synchronized. The sample
-- uses different remote database schema definitions, which download different
-- sets of columns. It is kept simple just to make the sample as transparent 
-- as possible.
message '    ---- Employee...' type info to client
go
CREATE TABLE Employee (
    EmployeeID INT PRIMARY KEY,
    EmployeeName VARCHAR( 128 ),
    EmployeePhone VARCHAR( 20 ),
    EmployeeEMail VARCHAR( 128 ),
    LastModified TIMESTAMP DEFAULT TIMESTAMP
)
go
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 1, 'employee1', '555-123-1234', 'employee1@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 2, 'employee2', '555-231-2341', 'employee2@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 3, 'employee3', '555-323-3234', 'employee3@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 4, 'employee4', '555-423-4234', 'employee4@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 5, 'employee5', '555-523-5234', 'employee5@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 6, 'employee6', '555-623-6234', 'employee6@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 7, 'employee7', '555-723-7234', 'employee7@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 8, 'employee8', '555-823-8234', 'employee8@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
              VALUES ( 9, 'employee9', '555-923-9234', 'employee9@example.com');
INSERT INTO Employee ( EmployeeID, EmployeeName, EmployeePhone, EmployeeEMail ) 
            VALUES ( 10, 'employee10', '555-123-0234', 'employee10@example.com');
COMMIT
go


--
-- From here on it's all about schema upgrade.
--

-- On the first synchronization the RemoteID is uploaded and the CurrentVersion 
-- and TargetVersion are set by the defaults.
-- After that, this table is maintained by the administrator (TargetVersion) 
-- and the Remote (CurrentVersion).
message '    ---- SU_RemoteDatabaseVersion...' type info to client
go
CREATE TABLE SU_RemoteDatabaseVersion (
    RemoteID VARCHAR(128) NOT NULL,
    CurrentVersion  INT NOT NULL DEFAULT 0,
    TargetVersion INT NOT NULL DEFAULT 1,
    LastModified TIMESTAMP DEFAULT TIMESTAMP,
    PRIMARY KEY (RemoteID)
)
go

-- Each SQL Statement handles an upgrade between CurrentVersion and TargetVersion 
-- of the schema.
-- All statements for a single (CurrentVersion, TargetVersion) are executed in sequence.
-- This table is downloaded only.
message '    ---- SU_SQLStatement ...' type info to client
go
create table SU_SQLStatement  (
    SqlID INT PRIMARY KEY DEFAULT AUTOINCREMENT,
    CurrentVersion INT,
    TargetVersion INT,
    Sequence INT,
    SqlStatement varchar(1024),
    LastModified TIMESTAMP DEFAULT TIMESTAMP,
)
go

message '  ---- Adding schema upgrade synchronization scripts...' type info to client
go

-- Version Schema synchronization scripts
-- Set the initial values for this remote ID if this is the first sync of the schema
call ml_add_table_script(
    'VersionSchema',
    'SU_RemoteDatabaseVersion',
    'begin_synchronization',
    'CALL insert_remote_id( {ml s.remote_id} )' 
)
go

CREATE PROCEDURE insert_remote_id(IN @RemoteID VARCHAR(128) )
BEGIN
    If ( SELECT COUNT(*) 
	 FROM SU_RemoteDatabaseVersion 
	 WHERE RemoteID = @RemoteID ) = 0 
    THEN
	INSERT INTO SU_RemoteDatabaseVersion( RemoteID) VALUES (@RemoteID)
    END IF
END
go


-- Upload update for remote database version: only the current version is changed.
call ml_add_table_script(
    'VersionSchema',
    'SU_RemoteDatabaseVersion',
    'upload_update',
    'UPDATE SU_RemoteDatabaseVersion SET CurrentVersion = {ml r.CurrentVersion} WHERE RemoteID = {ml s.remote_id}' 
)
go

call ml_add_column( 'VersionSchema', 'SU_RemoteDatabaseVersion', 'RemoteID', 'VARCHAR(128)')
go
call ml_add_column( 'VersionSchema', 'SU_RemoteDatabaseVersion', 'CurrentVersion', 'INT')
go
call ml_add_column( 'VersionSchema', 'SU_RemoteDatabaseVersion', 'TargetVersion', 'INT')
go


-- Download new schema information as it becomes available.
call ml_add_table_script(
    'VersionSchema',
    'SU_RemoteDatabaseVersion',
    'download_cursor',
    'SELECT RemoteID, CurrentVersion, TargetVersion FROM SU_RemoteDatabaseVersion WHERE RemoteID = {ml s.remote_id} AND LastModified >= {ml s.last_table_download}'
)
go

call ml_add_table_script(
    'VersionSchema',
    'SU_SQLStatement',
    'download_cursor',
    'SELECT SqlID, CurrentVersion, TargetVersion, Sequence, SqlStatement FROM SU_SQLStatement WHERE LastModified >= {ml s.last_table_download}'
)
go
